-- ========================================================================== --
-- 										 EskaQuestTracker                                       --
-- @Author   : Skamer <https://mods.curse.com/members/DevSkamer>              --
-- @Website  : https://wow.curseforge.com/projects/eska-quest-tracker         --
-- ========================================================================== --
Scorpio            "EskaQuestTracker.Classes.Frame"                           ""
-- ========================================================================== --
namespace "EQT"
import "System.Reflector"
-- ========================================================================== --
class "Frame"
  event "OnDrawRequest"
  event "OnWidthChanged"
  event "OnHeightChanged"
  event "OnSizeChanged"
  -- ======================================================================== --
  -- Handlers
  -- ======================================================================== --
  local function UpdateHeight(self, new, old, prop)
    if self.frame then
      self.frame:SetHeight(new)
    end
    return OnHeightChanged(self, new, old)
  end

  local function UpdateWidth(self, new, old, prop)
    if self.frame then
      self.frame:SetWidth(new)
    end
    return OnWidthChanged(self, new, old)
  end

  local function OnDrawRequestHandler(self)
    if not self.needToBeRedraw then
      self.needToBeRedraw = true
      Scorpio.Delay(0.25, function()
          local aborted = false
          if ObjectIsInterface(self, IReusable) and self.isReusable then
            aborted = true
          end

          if self.Draw and not aborted then self:Draw()  end
          self.needToBeRedraw = false
      end)
    end

  end
  -- ======================================================================== --
  -- Methods                                                                  --
  -- ======================================================================== --
  __Arguments__ { Number }
  function SetWidth(self, width)
    self.width = width
    return OnSizeChanged(self, width, self.height)
  end

  __Arguments__{}
  function GetWidth(self)
    return self.width
  end

  __Arguments__{ Number }
  function SetHeight(self, height)
    self.height = height
    return OnSizeChanged(self, self.width, height)
  end

  __Arguments__{ Number, Number }
  function SetSize(self, width, height)
    self.width = width
    self.height = height
    return OnSizeChanged(self, width, height)
  end

  __Arguments__{}
  function GetSize(self)
    return self.width, self.height
  end


  function SetParent(self, parent)
    self.frame:SetParent(parent)
  end

  function GetParent(self)
    self.frame:GetParent()
  end


  __Arguments__{}
  function ClearAllPoints(self)
    self.frame:ClearAllPoints()
  end

  __Arguments__{}
  function Show(self)
    self.frame:Show()
  end

  __Arguments__{}
  function Hide(self)
    self.frame:Hide()
  end

  __Arguments__{}
  function IsShown(self)
    return self.frame:IsShown()
  end
  -- ======================================================================== --
  -- Properties
  -- ======================================================================== --
  property "frame" { TYPE = Table }
  property "width" { TYPE = Number, HANDLER = UpdateWidth }
  property "height" { TYPE = Number, HANDLER = UpdateHeight }
  property "baseHeight" { TYPE = Number }
  property "needToBeRedraw" { TYPE = Boolean, DEFAULT = false }

  function Frame(self)
    self.OnDrawRequest = self.OnDrawRequest + OnDrawRequestHandler
  end

endclass "Frame"
